#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import click

from newschimp import renderer, sender
from newschimp.social import fb, gg, lanyrd
from newschimp.cli import cli_group
from newschimp.utils import ComplexCLI, load_settings


def create_newsletter(settings):
    """Newsletter creation based on config and env variables"""
    context = {}
    fb_posts = fb.get_posts(settings, os.environ.get('FACEBOOK_TOKEN'), None)
    context['fb'] = fb.curate(fb_posts)
    ggroup_posts = gg.get_posts(settings, None)
    context['gg'] = gg.curate(ggroup_posts)
    context['meetups'] = lanyrd.meetup_loop(settings)
    renderer.render_files(settings, None, context)
    click.confirm(
        'Content is rendered, would you like to send it now?', abort=True)
    sender.new_campaign(settings, os.environ.get('MAILCHIMP_KEY'))


cli_group.add_command(fb.cli)
cli_group.add_command(gg.cli)
cli_group.add_command(lanyrd.cli)

@cli_group.command(cls=ComplexCLI, invoke_without_command=True)
@click.option('--config', help='Custom config file', type=click.Path(
    exists=True, file_okay=True, resolve_path=True), default='config.yaml')
@click.pass_context
def main(ctx, config):
    ctx.obj['SETTINGS'] = load_settings(config)
    if ctx.invoked_subcommand is None:
        create_newsletter(ctx.obj['SETTINGS'])


if __name__ == '__main__':
    main(obj={})



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os
import sys
from pprint import pprint

import click
import requests as req

FACEBOOK_GROUP_URL = 'https://graph.facebook.com/v2.0/{}/feed'
LOGGER = logging.getLogger(__name__)


def get_posts(settings, token, group):
    try:
        group_id = group if group else settings['facebook_group_id']
    except KeyError:
        LOGGER.error('Facebook Group id not defined')
        sys.exit()
    group_uri = FACEBOOK_GROUP_URL.format(group_id)
    payload = {
        'limit': 10000,
        'access_token': token,
        'since': settings['since']
    }
    posts = req.get(group_uri, params=payload)
    try:
        return posts.json()['data']
    except KeyError:
        LOGGER.error('Facebook API error:')
        LOGGER.error(posts.json()['error'])
        sys.exit()


def score(post):
    try:
        temp_score = len(post['likes']['data'])
    except KeyError:
        temp_score = 0
    try:
        for comment in post['comments']['data']:
            temp_score += comment['like_count']
    finally:
        return temp_score


def curate(posts, count=5):
    active = [post for post in posts if score(post) > 1]
    best = sorted(active, key=lambda post: score(post))[-count:]
    for post in best:
        summary = {
            'fb_url': post['actions'][0]['link'],
            'from': post['from']['name'],
        }
        if 'message' in post:
            summary['message'] = post['message']
        if 'link' in post:
            summary['link_name'] = post['name']
            summary['link_url'] = post['link']
        yield summary


@click.command(short_help='Facebook curator')
@click.option('--group', help='Group ID')
@click.option('--token', help='Facebook token', envvar='FACEBOOK_TOKEN')
@click.pass_context
def cli(ctx, token, group):
    monthly_posts = get_posts(ctx.obj['SETTINGS'], token, group)
    best_posts = curate(monthly_posts)
    for post in best_posts:
        pprint(post)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os
import sys

import click
from lxml.html import fromstring
from jinja2 import Environment, FileSystemLoader
from typogrify.filters import typogrify

from .cli import cli_group

LOGGER = logging.getLogger(__name__)


def render_files(settings, template, context={}):
    env = Environment(loader=FileSystemLoader(os.path.dirname(__file__)))
    if 'context' in settings:
        settings['context'].update(context)
    else:
        settings['context'] = context
    full_context = {k.upper(): v for k, v in settings['context'].items()}
    try:
        used_template = template if template else settings['template']
    except KeyError:
        LOGGER.error('No template defined')
        sys.exit()
    html_output = env.get_template(settings['template']).render(**full_context)
    text_output = fromstring(html_output).text_content()
    typo_output = typogrify(html_output)
    if settings.get('html_output'):
        with open(settings['html_output'], 'w') as html:
            html.write(typo_output)
        if settings.get('text_output'):
            with open(settings['text_output'], 'w') as text:
                text.write(text_output)
    else:
        print(text_output)


@cli_group.command(short_help='HTML and text email rendering')
@click.option('--template', help='Template file', type=click.Path(
    exists=True, file_okay=True, resolve_path=True))
@click.pass_context
def cli(ctx, template):
    render_files(ctx.obj['SETTINGS'], template)



# -*- coding: utf-8 -*-
import os
import sys

import click
import yaml

# TODO: Logging

# TODO: The fuck
CZ_MONTHS = {
    'března': 3,
    'dubna': 4,
    'května': 5,
    'června': 6,
}

UTIL_FILES = ('__init__.py', 'utils.py', 'cli.py', 'core.py')


def readable(file_name):
    if os.path.isfile(file_name) and os.access(file_name, os.R_OK):
        return True
    print(file_name, ' is not readable or does not exist!')
    return False


def load_settings(config_file):
    if not readable(config_file):
        exit()
    with open(config_file, 'r') as config:
        settings = yaml.load(config)
        return settings


cmd_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                          'social'))
root_folder = os.path.abspath(os.path.dirname(__file__))

class ComplexCLI(click.MultiCommand):

    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(cmd_folder):
            if filename.endswith('.py') and filename not in UTIL_FILES:
                rv.append(filename[:-3])
        for filename in os.listdir(root_folder):
            if filename.endswith('.py') and filename not in UTIL_FILES:
                rv.append(filename[:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        try:
            mod = __import__('newschimp.social.' + name, None, None, ['cli'])
            return mod.cli
        except ImportError:
            pass
        try:
            mod = __import__('newschimp.' + name, None, None, ['cli'])
        except ImportError:
            return
        return mod.cli



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import asyncio
from datetime import datetime
from pprint import pprint

import click
import requests as req
from lxml.html import fromstring

LANYRD_BASE = 'http://lanyrd.com/'
LANYRD_URL = LANYRD_BASE + 'search/?q={}'


@asyncio.coroutine
def get_search(event):
    search_url = LANYRD_URL.format(event)
    search_results = req.get(search_url)
    frontpage = fromstring(search_results.text)
    frontpage.make_links_absolute(LANYRD_BASE)
    results = frontpage.xpath('//li[@class="s-conference s-result"]')
    return results


@asyncio.coroutine
def get_next_meetup(event, month):
    search = yield from get_search(event)
    for event in search:
        meetup_date = event.xpath('.//p[@class="date"]/abbr'
            )[0].attrib['title']
        parsed_date = datetime.strptime(meetup_date, '%Y-%m-%d')
        if parsed_date.month < month:
            continue
        event_node = event.xpath('.//h3/a')[0]
        return {
            'name': event_node.text,
            'url': event_node.attrib['href']
        }


@asyncio.coroutine
def scrape_meetups(settings):
    results = []
    for event in settings['events']:
        meetup = yield from get_next_meetup(event, settings['month'])
        results.append(meetup)
    return results


def meetup_loop(settings):
    loop = asyncio.get_event_loop()
    results = loop.run_until_complete(scrape_meetups(settings))
    loop.close()
    return results


@click.command(short_help='Meetup listing')
@click.pass_context
def cli(ctx):
    meetups = meetup_loop(ctx.obj['SETTINGS'])
    for meetup in meetups:
        pprint(meetup)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import time
import sys
from datetime import date
from pprint import pprint

import click
from lxml.html import fromstring
from selenium import webdriver

CZ_MONTHS = {
    'března': 3,
    'dubna': 4,
    'května': 5,
    'června': 6,
}
GOOGLE_GROUP_BASE = 'https://groups.google.com/forum/'
GOOGLE_GROUP_URL = GOOGLE_GROUP_BASE + '#!forum/{}'
LOGGER = logging.getLogger(__name__)


def date_parse(raw_date):
    tokens = raw_date.split()
    day = int(tokens[1].strip('.'))
    month = CZ_MONTHS[tokens[2]]
    year = int(tokens[3])
    return date(year, month, day)


def thread_to_dict(thread):
    parsed = {'name': thread.xpath('.//a[@class="GIEUOX-DPL"]')[0].text}
    parsed['url'] = thread.xpath('.//a[@class="GIEUOX-DPL"]')[0].attrib['href']
    raw_last_change = thread.xpath('.//span[@class="GIEUOX-DOQ"]/span'
        )[0].attrib['title']
    last_change = date_parse(raw_last_change)
    parsed['month'] = last_change.month
    info = thread.xpath('.//span[@class="GIEUOX-DOQ"]')
    parsed['seen'] = int(info[1].text.split()[0])
    parsed['posts'] = int(info[0].text.split()[0])
    return parsed


def get_posts(settings, group):
    try:
        group_id = group if group else settings['google_group_name']
    except KeyError:
        LOGGER.error('Google Group name not defined')
        sys.exit()
    group_url = GOOGLE_GROUP_URL.format(group_id)
    browser = webdriver.PhantomJS()
    browser.set_window_size(1024, 768)
    browser.get(group_url)
    time.sleep(5)
    frontpage = fromstring(browser.page_source)
    browser.quit()
    frontpage.make_links_absolute(GOOGLE_GROUP_BASE)
    html_threads = frontpage.xpath('//div[@class="GIEUOX-DEQ"]')
    threads = (thread_to_dict(thread) for thread in html_threads)
    return [
        thread for thread in threads if thread['month'] >= settings['month']]


def score(post):
    return post['seen'] + post['posts']


def curate(posts, count=3):
    active = [post for post in posts if score(post) > 1]
    best = sorted(active, key=lambda post: score(post))[-count:]
    for post in best:
        summary = {
            'name': post['name'],
            'url': post['url'],
        }
        yield summary


@click.command()
@click.option('--group', help='Group ID')
@click.pass_context
def cli(ctx, group):
    """Google Groups curator"""
    monthly_posts = get_posts(ctx.obj['SETTINGS'], group)
    best_posts = curate(monthly_posts)
    for post in best_posts:
        pprint(post)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import os

import click
import mailchimp

from .cli import cli_group

LOGGER = logging.getLogger(__name__)


def new_campaign(settings, key):
    api = mailchimp.Mailchimp(key)
    text_file = open(settings['text_output'], 'r')
    html_file = open(settings['html_output'], 'r')
    resp = api.campaigns.create(
        'regular',
        {
            'list_id': settings['mail_list'],
            'subject': settings['subject'],
            'from_email': settings['sender']['email'],
            'from_name': settings['sender']['name'],
            'to_name': settings['reciever'],
        },
        {
            'html': html_file.read(),
            'text': text_file.read(),
        })
    if resp['status'] == 'error':
        LOGGER.error('MailChimp API error: ')
        LOGGER.error(' '.join((resp['code'], resp['error'])))
    text_file.close()
    html_file.close()


@cli_group.command(short_help='Campaign creation')
@click.option('--key', help='Mailchimp API key', envvar='MAILCHIMP_KEY')
@click.pass_context
def cli(ctx, key):
    new_campaign(ctx.obj['SETTINGS'], key)



#!/usr/bin/env python
from setuptools import setup

with open('README.rst') as f:
    readme = f.read()

setup(
    name='NewsChimp',
    packages=['newschimp', 'newschimp.social'],
    version='0.1.3',
    description='Newsletter generator for MailChimp',
    long_description=readme,
    author='Martin Putniorz',
    author_email='mputniorz@gmail.cpm',
    url='https://github.com/sputnikus/newschimp',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        'Topic :: Communications :: Email',
    ],
    install_requires=[
        'Jinja2>=2.7',
        'PyYAML>=3.11',
        'click>=2.0',
        'lxml>=3.3',
        'mailchimp>=2.0',
        'requests>=2.3',
        'selenium>=2.42',
        'smartypants>=1.8',
        'typogrify>=2.0',
    ],
    scripts=['bin/chimpgen'],
    license='BSD',
    keywords = ['newsletter', 'mail', 'mailchimp']
)
